@@ -9,7 +9,7 @@ from tempfile import mkdtemp
 from twisted.trial import unittest
 
 from scrapy.utils.python import retry_on_eintr
-from scrapy.utils.test import get_pythonpath
+from scrapy.utils.test import get_testenv
 
 class ProjectTest(unittest.TestCase):
     project_name = 'testproject'
@@ -19,8 +19,7 @@ class ProjectTest(unittest.TestCase):
         self.cwd = self.temp_path
         self.proj_path = join(self.temp_path, self.project_name)
         self.proj_mod_path = join(self.proj_path, self.project_name)
-        self.env = os.environ.copy()
-        self.env['PYTHONPATH'] = get_pythonpath()
+        self.env = get_testenv()
 
     def tearDown(self):
         rmtree(self.temp_path)
@@ -112,6 +111,10 @@ class GenspiderCommandTest(CommandTest):
         self.assertEqual(0, self.call('genspider', '--dump=basic'))
         self.assertEqual(0, self.call('genspider', '-d', 'basic'))
 
+    def test_same_name_as_project(self):
+        self.assertEqual(2, self.call('genspider', self.project_name))
+        assert not exists(join(self.proj_mod_path, 'spiders', '%s.py' % self.project_name))
+
 
 class MiscCommandsTest(CommandTest):
 
@@ -171,3 +174,64 @@ from scrapy.spider import BaseSpider
         log = p.stderr.read()
         self.assert_("Unable to load" in log)
 
+
+class ParseCommandTest(CommandTest):
+
+    def setUp(self):
+        super(ParseCommandTest, self).setUp()
+        self.spider_name = 'parse_spider'
+        fname = abspath(join(self.proj_mod_path, 'spiders', 'myspider.py'))
+        with open(fname, 'w') as f:
+            f.write("""
+from scrapy import log
+from scrapy.spider import BaseSpider
+from scrapy.item import Item
+
+class MySpider(BaseSpider):
+    name = '{0}'
+
+    def parse(self, response):
+        if getattr(self, 'test_arg', None):
+            self.log('It Works!')
+        return [Item()]
+""".format(self.spider_name))
+
+        fname = abspath(join(self.proj_mod_path, 'pipelines.py'))
+        with open(fname, 'w') as f:
+            f.write("""
+from scrapy import log
+
+class MyPipeline(object):
+    component_name = 'my_pipeline'
+
+    def process_item(self, item, spider):
+        log.msg('It Works!')
+        return item
+""")
+
+        fname = abspath(join(self.proj_mod_path, 'settings.py'))
+        with open(fname, 'a') as f:
+            f.write("""
+ITEM_PIPELINES = ['{0}.pipelines.MyPipeline']
+""".format(self.project_name))
+
+    def test_spider_arguments(self):
+        p = self.proc('parse', '--spider', self.spider_name, '-a', 'test_arg=1',
+                '-c', 'parse', 'http://scrapinghub.com')
+        log = p.stderr.read()
+        self.assert_("[parse_spider] DEBUG: It Works!" in log, log)
+
+    def test_pipelines(self):
+        p = self.proc('parse', '--spider', self.spider_name, '--pipelines',
+                '-c', 'parse', 'http://scrapinghub.com')
+        log = p.stderr.read()
+        self.assert_("[scrapy] INFO: It Works!" in log, log)
+
+
+class BenchCommandTest(CommandTest):
+
+    def test_run(self):
+        p = self.proc('bench', '-s', 'LOGSTATS_INTERVAL=0.001',
+                '-s', 'CLOSESPIDER_TIMEOUT=0.01')
+        log = p.stderr.read()
+        self.assert_('INFO: Crawled' in log, log)
