@@ -1,11 +1,11 @@
 import re
 import unittest
-
 from scrapy.http import HtmlResponse
 from scrapy.link import Link
 from scrapy.contrib.linkextractors.sgml import SgmlLinkExtractor, BaseSgmlLinkExtractor
 from scrapy.tests import get_testdata
 
+
 class LinkExtractorTestCase(unittest.TestCase):
     def test_basic(self):
         html = """<html><head><title>Page title<title>
@@ -20,9 +20,9 @@ class LinkExtractorTestCase(unittest.TestCase):
 
         lx = BaseSgmlLinkExtractor()  # default: tag=a, attr=href
         self.assertEqual(lx.extract_links(response),
-                         [Link(url='http://example.org/somepage/item/12.html', text='Item 12'), 
+                         [Link(url='http://example.org/somepage/item/12.html', text='Item 12'),
                           Link(url='http://example.org/about.html', text='About us'),
-                          Link(url='http://example.org/othercat.html', text='Other category'), 
+                          Link(url='http://example.org/othercat.html', text='Other category'),
                           Link(url='http://example.org/', text='>>'),
                           Link(url='http://example.org/', text='')])
 
@@ -54,8 +54,9 @@ class LinkExtractorTestCase(unittest.TestCase):
         html = """<body><p><a href="item/12.html">Wrong: \xed</a></p></body></html>"""
         response = HtmlResponse("http://www.example.com", body=html, encoding='utf-8')
         lx = BaseSgmlLinkExtractor()
-        self.assertEqual(lx.extract_links(response),
-             [Link(url='http://www.example.com/item/12.html', text=u'Wrong: \ufffd')])
+        self.assertEqual(lx.extract_links(response), [
+            Link(url='http://www.example.com/item/12.html', text=u'Wrong: \ufffd'),
+        ])
 
     def test_extraction_encoding(self):
         body = get_testdata('link_extractor', 'linkextractor_noenc.html')
@@ -65,17 +66,20 @@ class LinkExtractorTestCase(unittest.TestCase):
         response_latin1 = HtmlResponse(url='http://example.com/latin1', body=body)
 
         lx = BaseSgmlLinkExtractor()
-        self.assertEqual(lx.extract_links(response_utf8),
-            [ Link(url='http://example.com/sample_%C3%B1.html', text=''),
-              Link(url='http://example.com/sample_%E2%82%AC.html', text='sample \xe2\x82\xac text'.decode('utf-8')) ])
+        self.assertEqual(lx.extract_links(response_utf8), [
+            Link(url='http://example.com/sample_%C3%B1.html', text=''),
+            Link(url='http://example.com/sample_%E2%82%AC.html', text='sample \xe2\x82\xac text'.decode('utf-8')),
+        ])
 
-        self.assertEqual(lx.extract_links(response_noenc),
-            [ Link(url='http://example.com/sample_%C3%B1.html', text=''),
-              Link(url='http://example.com/sample_%E2%82%AC.html', text='sample \xe2\x82\xac text'.decode('utf-8')) ])
+        self.assertEqual(lx.extract_links(response_noenc), [
+            Link(url='http://example.com/sample_%C3%B1.html', text=''),
+            Link(url='http://example.com/sample_%E2%82%AC.html', text='sample \xe2\x82\xac text'.decode('utf-8')),
+        ])
 
-        self.assertEqual(lx.extract_links(response_latin1),
-            [ Link(url='http://example.com/sample_%F1.html', text=''),
-              Link(url='http://example.com/sample_%E1.html', text='sample \xe1 text'.decode('latin1')) ])
+        self.assertEqual(lx.extract_links(response_latin1), [
+            Link(url='http://example.com/sample_%F1.html', text=''),
+            Link(url='http://example.com/sample_%E1.html', text='sample \xe1 text'.decode('latin1')),
+        ])
 
     def test_matches(self):
         url1 = 'http://lotsofstuff.com/stuff1/index'
@@ -85,6 +89,19 @@ class LinkExtractorTestCase(unittest.TestCase):
         self.assertEqual(lx.matches(url1), True)
         self.assertEqual(lx.matches(url2), True)
 
+    def test_link_nofollow(self):
+        html = """
+        <a href="page.html?action=print" rel="nofollow">Printer-friendly page</a>
+        <a href="about.html">About us</a>
+        """
+        response = HtmlResponse("http://example.org/page.html", body=html)
+        lx = SgmlLinkExtractor()
+        self.assertEqual([link for link in lx.extract_links(response)], [
+            Link(url='http://example.org/page.html?action=print', text=u'Printer-friendly page', nofollow=True),
+            Link(url='http://example.org/about.html', text=u'About us', nofollow=False),
+        ])
+
+
 class SgmlLinkExtractorTestCase(unittest.TestCase):
     def setUp(self):
         body = get_testdata('link_extractor', 'sgml_linkextractor.html')
@@ -99,62 +116,71 @@ class SgmlLinkExtractorTestCase(unittest.TestCase):
         '''Test the extractor's behaviour among different situations'''
 
         lx = SgmlLinkExtractor()
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://example.com/sample1.html', text=u''),
-              Link(url='http://example.com/sample2.html', text=u'sample 2'),
-              Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
-              Link(url='http://www.google.com/something', text=u'') ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://example.com/sample1.html', text=u''),
+            Link(url='http://example.com/sample2.html', text=u'sample 2'),
+            Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+            Link(url='http://www.google.com/something', text=u''),
+        ])
 
         lx = SgmlLinkExtractor(allow=('sample', ))
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://example.com/sample1.html', text=u''),
-              Link(url='http://example.com/sample2.html', text=u'sample 2'),
-              Link(url='http://example.com/sample3.html', text=u'sample 3 text') ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://example.com/sample1.html', text=u''),
+            Link(url='http://example.com/sample2.html', text=u'sample 2'),
+            Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+        ])
 
         lx = SgmlLinkExtractor(allow=('sample', ), unique=False)
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://example.com/sample1.html', text=u''),
-              Link(url='http://example.com/sample2.html', text=u'sample 2'),
-              Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
-              Link(url='http://example.com/sample3.html', text=u'sample 3 repetition') ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://example.com/sample1.html', text=u''),
+            Link(url='http://example.com/sample2.html', text=u'sample 2'),
+            Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+            Link(url='http://example.com/sample3.html', text=u'sample 3 repetition'),
+        ])
 
         lx = SgmlLinkExtractor(allow=('sample', ))
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://example.com/sample1.html', text=u''),
-              Link(url='http://example.com/sample2.html', text=u'sample 2'),
-              Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
-              ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://example.com/sample1.html', text=u''),
+            Link(url='http://example.com/sample2.html', text=u'sample 2'),
+            Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+        ])
 
         lx = SgmlLinkExtractor(allow=('sample', ), deny=('3', ))
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://example.com/sample1.html', text=u''),
-              Link(url='http://example.com/sample2.html', text=u'sample 2') ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://example.com/sample1.html', text=u''),
+            Link(url='http://example.com/sample2.html', text=u'sample 2'),
+        ])
 
         lx = SgmlLinkExtractor(allow_domains=('google.com', ))
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://www.google.com/something', text=u'') ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://www.google.com/something', text=u''),
+        ])
 
     def test_extraction_using_single_values(self):
         '''Test the extractor's behaviour among different situations'''
 
         lx = SgmlLinkExtractor(allow='sample')
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://example.com/sample1.html', text=u''),
-              Link(url='http://example.com/sample2.html', text=u'sample 2'),
-              Link(url='http://example.com/sample3.html', text=u'sample 3 text') ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://example.com/sample1.html', text=u''),
+            Link(url='http://example.com/sample2.html', text=u'sample 2'),
+            Link(url='http://example.com/sample3.html', text=u'sample 3 text'),
+        ])
 
         lx = SgmlLinkExtractor(allow='sample', deny='3')
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://example.com/sample1.html', text=u''),
-              Link(url='http://example.com/sample2.html', text=u'sample 2') ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://example.com/sample1.html', text=u''),
+            Link(url='http://example.com/sample2.html', text=u'sample 2'),
+        ])
 
         lx = SgmlLinkExtractor(allow_domains='google.com')
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://www.google.com/something', text=u'') ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://www.google.com/something', text=u''),
+        ])
 
         lx = SgmlLinkExtractor(deny_domains='example.com')
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://www.google.com/something', text=u'') ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://www.google.com/something', text=u''),
+        ])
 
     def test_matches(self):
         url1 = 'http://lotsofstuff.com/stuff1/index'
@@ -176,8 +202,9 @@ class SgmlLinkExtractorTestCase(unittest.TestCase):
         self.assertEqual(lx.matches(url1), False)
         self.assertEqual(lx.matches(url2), True)
 
-        lx = SgmlLinkExtractor(allow=('blah1', ), deny=('blah2', ),
-            allow_domains=('blah1.com', ), deny_domains=('blah2.com', ))
+        lx = SgmlLinkExtractor(allow=('blah1',), deny=('blah2',),
+                               allow_domains=('blah1.com',),
+                               deny_domains=('blah2.com',))
         self.assertEqual(lx.matches('http://blah1.com/blah1'), True)
         self.assertEqual(lx.matches('http://blah1.com/blah2'), False)
         self.assertEqual(lx.matches('http://blah2.com/blah1'), False)
@@ -185,9 +212,10 @@ class SgmlLinkExtractorTestCase(unittest.TestCase):
 
     def test_restrict_xpaths(self):
         lx = SgmlLinkExtractor(restrict_xpaths=('//div[@id="subwrapper"]', ))
-        self.assertEqual([link for link in lx.extract_links(self.response)],
-            [ Link(url='http://example.com/sample1.html', text=u''),
-              Link(url='http://example.com/sample2.html', text=u'sample 2') ])
+        self.assertEqual([link for link in lx.extract_links(self.response)], [
+            Link(url='http://example.com/sample1.html', text=u''),
+            Link(url='http://example.com/sample2.html', text=u'sample 2'),
+        ])
 
     def test_restrict_xpaths_encoding(self):
         """Test restrict_xpaths with encodings"""
@@ -202,16 +230,42 @@ class SgmlLinkExtractorTestCase(unittest.TestCase):
         </body></html>"""
         response = HtmlResponse("http://example.org/somepage/index.html", body=html, encoding='windows-1252')
 
-        lx = SgmlLinkExtractor(restrict_xpaths="//div[@class='links']") 
+        lx = SgmlLinkExtractor(restrict_xpaths="//div[@class='links']")
         self.assertEqual(lx.extract_links(response),
                          [Link(url='http://example.org/about.html', text=u'About us\xa3')])
 
+    def test_restrict_xpaths_concat_in_handle_data(self):
+        """html entities cause SGMLParser to call handle_data hook twice"""
+        body = """<html><body><div><a href="/foo">&gt;\xbe\xa9&lt;\xb6\xab</a></body></html>"""
+        response = HtmlResponse("http://example.org", body=body, encoding='gb18030')
+        lx = SgmlLinkExtractor(restrict_xpaths="//div")
+        self.assertEqual(lx.extract_links(response),
+                         [Link(url='http://example.org/foo', text=u'>\u4eac<\u4e1c',
+                               fragment='', nofollow=False)])
+
+    def test_encoded_url(self):
+        body = """<html><body><div><a href="?page=2">BinB</a></body></html>"""
+        response = HtmlResponse("http://known.fm/AC%2FDC/", body=body, encoding='utf8')
+        lx = SgmlLinkExtractor()
+        self.assertEqual(lx.extract_links(response), [
+            Link(url='http://known.fm/AC%2FDC/?page=2', text=u'BinB', fragment='', nofollow=False),
+        ])
+
+    def test_encoded_url_in_restricted_xpath(self):
+        body = """<html><body><div><a href="?page=2">BinB</a></body></html>"""
+        response = HtmlResponse("http://known.fm/AC%2FDC/", body=body, encoding='utf8')
+        lx = SgmlLinkExtractor(restrict_xpaths="//div")
+        self.assertEqual(lx.extract_links(response), [
+            Link(url='http://known.fm/AC%2FDC/?page=2', text=u'BinB', fragment='', nofollow=False),
+        ])
+
     def test_deny_extensions(self):
         html = """<a href="page.html">asd</a> and <a href="photo.jpg">"""
         response = HtmlResponse("http://example.org/", body=html)
         lx = SgmlLinkExtractor()
-        self.assertEqual(lx.extract_links(response),
-            [Link(url='http://example.org/page.html', text=u'asd')])
+        self.assertEqual(lx.extract_links(response), [
+            Link(url='http://example.org/page.html', text=u'asd'),
+        ])
 
     def test_process_value(self):
         """Test restrict_xpaths with encodings"""
@@ -235,11 +289,10 @@ class SgmlLinkExtractorTestCase(unittest.TestCase):
         <body><p><a href="item/12.html">Item 12</a></p>
         </body></html>"""
         response = HtmlResponse("http://example.org/somepage/index.html", body=html)
-        lx = SgmlLinkExtractor(restrict_xpaths="//p") 
+        lx = SgmlLinkExtractor(restrict_xpaths="//p")
         self.assertEqual(lx.extract_links(response),
                          [Link(url='http://otherdomain.com/base/item/12.html', text='Item 12')])
 
 
-
 if __name__ == "__main__":
     unittest.main()
