@@ -1,18 +1,27 @@
 import os
+import twisted
 
 from twisted.trial import unittest
 from twisted.protocols.policies import WrappingFactory
 from twisted.python.filepath import FilePath
-from twisted.internet import reactor, defer
+from twisted.internet import reactor, defer, error
 from twisted.web import server, static, util, resource
 from twisted.web.test.test_webclient import ForeverTakingResource, \
         NoLengthResource, HostHeaderResource, \
         PayloadResource, BrokenDownloadResource
+from twisted.protocols.ftp import FTPRealm, FTPFactory
+from twisted.cred import portal, checkers, credentials
+from twisted.protocols.ftp import FTPClient, ConnectionLost
 from w3lib.url import path_to_file_uri
 
+from scrapy import twisted_version
 from scrapy.core.downloader.handlers.file import FileDownloadHandler
-from scrapy.core.downloader.handlers.http import HttpDownloadHandler
+from scrapy.core.downloader.handlers.http import HTTPDownloadHandler, HttpDownloadHandler
+from scrapy.core.downloader.handlers.http10 import HTTP10DownloadHandler
+from scrapy.core.downloader.handlers.http11 import HTTP11DownloadHandler
 from scrapy.core.downloader.handlers.s3 import S3DownloadHandler
+from scrapy.core.downloader.handlers.ftp import FTPDownloadHandler
+
 from scrapy.spider import BaseSpider
 from scrapy.http import Request
 from scrapy.settings import Settings
@@ -46,6 +55,8 @@ class FileTestCase(unittest.TestCase):
 
 class HttpTestCase(unittest.TestCase):
 
+    download_handler_cls = HTTPDownloadHandler
+
     def setUp(self):
         name = self.mktemp()
         os.mkdir(name)
@@ -53,6 +64,7 @@ class HttpTestCase(unittest.TestCase):
         r = static.File(name)
         r.putChild("redirect", util.Redirect("/file"))
         r.putChild("wait", ForeverTakingResource())
+        r.putChild("hang-after-headers", ForeverTakingResource(write=True))
         r.putChild("nolength", NoLengthResource())
         r.putChild("host", HostHeaderResource())
         r.putChild("payload", PayloadResource())
@@ -61,10 +73,14 @@ class HttpTestCase(unittest.TestCase):
         self.wrapper = WrappingFactory(self.site)
         self.port = reactor.listenTCP(0, self.wrapper, interface='127.0.0.1')
         self.portno = self.port.getHost().port
-        self.download_request = HttpDownloadHandler(Settings()).download_request
+        self.download_handler = self.download_handler_cls(Settings())
+        self.download_request = self.download_handler.download_request
 
+    @defer.inlineCallbacks
     def tearDown(self):
-        return self.port.stopListening()
+        yield self.port.stopListening()
+        if hasattr(self.download_handler, 'close'):
+            yield self.download_handler.close()
 
     def getURL(self, path):
         return "http://127.0.0.1:%d/%s" % (self.portno, path)
@@ -97,10 +113,18 @@ class HttpTestCase(unittest.TestCase):
         d.addCallback(self.assertEquals, 302)
         return d
 
+    @defer.inlineCallbacks
     def test_timeout_download_from_spider(self):
-        request = Request(self.getURL('wait'), meta=dict(download_timeout=0.000001))
-        d = self.download_request(request, BaseSpider('foo'))
-        return self.assertFailure(d, defer.TimeoutError)
+        spider = BaseSpider('foo')
+        meta = {'download_timeout': 0.2}
+        # client connects but no data is received
+        request = Request(self.getURL('wait'), meta=meta)
+        d = self.download_request(request, spider)
+        yield self.assertFailure(d, defer.TimeoutError, error.TimeoutError)
+        # client connects, server send headers and some body bytes but hangs
+        request = Request(self.getURL('hang-after-headers'), meta=meta)
+        d = self.download_request(request, spider)
+        yield self.assertFailure(d, defer.TimeoutError, error.TimeoutError)
 
     def test_host_header_not_in_request_headers(self):
         def _test(response):
@@ -132,6 +156,23 @@ class HttpTestCase(unittest.TestCase):
         return d
 
 
+class DeprecatedHttpTestCase(HttpTestCase):
+    """HTTP 1.0 test case"""
+    download_handler_cls = HttpDownloadHandler
+
+
+class Http10TestCase(HttpTestCase):
+    """HTTP 1.0 test case"""
+    download_handler_cls = HTTP10DownloadHandler
+
+
+class Http11TestCase(HttpTestCase):
+    """HTTP 1.1 test case"""
+    download_handler_cls = HTTP11DownloadHandler
+    if 'http11' not in optional_features:
+        skip = 'HTTP1.1 not supported in twisted < 11.1.0'
+
+
 class UriResource(resource.Resource):
     """Return the full uri that was requested"""
 
@@ -143,16 +184,21 @@ class UriResource(resource.Resource):
 
 
 class HttpProxyTestCase(unittest.TestCase):
+    download_handler_cls = HTTPDownloadHandler
 
     def setUp(self):
         site = server.Site(UriResource(), timeout=None)
         wrapper = WrappingFactory(site)
         self.port = reactor.listenTCP(0, wrapper, interface='127.0.0.1')
         self.portno = self.port.getHost().port
-        self.download_request = HttpDownloadHandler(Settings()).download_request
+        self.download_handler = self.download_handler_cls(Settings())
+        self.download_request = self.download_handler.download_request
 
+    @defer.inlineCallbacks
     def tearDown(self):
-        return self.port.stopListening()
+        yield self.port.stopListening()
+        if hasattr(self.download_handler, 'close'):
+            yield self.download_handler.close()
 
     def getURL(self, path):
         return "http://127.0.0.1:%d/%s" % (self.portno, path)
@@ -177,6 +223,21 @@ class HttpProxyTestCase(unittest.TestCase):
         return self.download_request(request, BaseSpider('foo')).addCallback(_test)
 
 
+class DeprecatedHttpProxyTestCase(unittest.TestCase):
+    """Old deprecated reference to http10 downloader handler"""
+    download_handler_cls = HttpDownloadHandler
+
+
+class Http10ProxyTestCase(HttpProxyTestCase):
+    download_handler_cls = HTTP10DownloadHandler
+
+
+class Http11ProxyTestCase(HttpProxyTestCase):
+    download_handler_cls = HTTP11DownloadHandler
+    if 'http11' not in optional_features:
+        skip = 'HTTP1.1 not supported in twisted < 11.1.0'
+
+
 class HttpDownloadHandlerMock(object):
     def __init__(self, settings):
         pass
@@ -240,7 +301,7 @@ class S3TestCase(unittest.TestCase):
                 'AWS 0PN5J17HBGZHT7JJ3X82:thdUi9VAkzhkniLj96JIrOPGi0g=')
 
     def test_request_signing5(self):
-        # deletes an object from the 'johnsmith' bucket using the 
+        # deletes an object from the 'johnsmith' bucket using the
         # path-style and Date alternative.
         req = Request('s3://johnsmith/photos/puppy.jpg', \
                 method='DELETE', headers={
@@ -271,3 +332,86 @@ class S3TestCase(unittest.TestCase):
         httpreq = self.download_request(req, self.spider)
         self.assertEqual(httpreq.headers['Authorization'], \
                 'AWS 0PN5J17HBGZHT7JJ3X82:C0FlOtU8Ylb9KDTpZqYkZPX91iI=')
+
+class FTPTestCase(unittest.TestCase):
+
+    username = "scrapy"
+    password = "passwd"
+
+    if twisted_version < (10, 2, 0):
+        skip = "Twisted pre 10.2.0 doesn't allow to set home path other than /home"
+
+    def setUp(self):
+        # setup dirs and test file
+        self.directory = self.mktemp()
+        os.mkdir(self.directory)
+        userdir = os.path.join(self.directory, self.username)
+        os.mkdir(userdir)
+        FilePath(userdir).child('file.txt').setContent("I have the power!")
+
+        # setup server
+        realm = FTPRealm(anonymousRoot=self.directory, userHome=self.directory)
+        p = portal.Portal(realm)
+        users_checker = checkers.InMemoryUsernamePasswordDatabaseDontUse()
+        users_checker.addUser(self.username, self.password)
+        p.registerChecker(users_checker, credentials.IUsernamePassword)
+        self.factory = FTPFactory(portal=p)
+        self.port = reactor.listenTCP(0, self.factory, interface="127.0.0.1")
+        self.portNum = self.port.getHost().port
+        self.download_handler = FTPDownloadHandler(Settings())
+        self.addCleanup(self.port.stopListening)
+
+    def _add_test_callbacks(self, deferred, callback=None, errback=None):
+        def _clean(data):
+            self.download_handler.client.transport.loseConnection()
+            return data
+        deferred.addCallback(_clean)
+        if callback:
+            deferred.addCallback(callback)
+        if errback:
+            deferred.addErrback(errback)
+        return deferred
+
+    def test_ftp_download_success(self):
+        request = Request(url="ftp://127.0.0.1:%s/file.txt" % self.portNum,
+                meta={"ftp_user": self.username, "ftp_password": self.password})
+        d = self.download_handler.download_request(request, None)
+
+        def _test(r):
+            self.assertEqual(r.status, 200)
+            self.assertEqual(r.body, 'I have the power!')
+            self.assertEqual(r.headers, {'Local Filename': [''], 'Size': [17]})
+        return self._add_test_callbacks(d, _test)
+
+    def test_ftp_download_notexist(self):
+        request = Request(url="ftp://127.0.0.1:%s/notexist.txt" % self.portNum,
+                meta={"ftp_user": self.username, "ftp_password": self.password})
+        d = self.download_handler.download_request(request, None)
+
+        def _test(r):
+            self.assertEqual(r.status, 404)
+        return self._add_test_callbacks(d, _test)
+
+    def test_ftp_local_filename(self):
+        local_fname = "/tmp/file.txt"
+        request = Request(url="ftp://127.0.0.1:%s/file.txt" % self.portNum,
+                meta={"ftp_user": self.username, "ftp_password": self.password, "ftp_local_filename": local_fname})
+        d = self.download_handler.download_request(request, None)
+
+        def _test(r):
+            self.assertEqual(r.body, local_fname)
+            self.assertEqual(r.headers, {'Local Filename': ['/tmp/file.txt'], 'Size': [17]})
+            self.assertTrue(os.path.exists(local_fname))
+            with open(local_fname) as f:
+                self.assertEqual(f.read(), "I have the power!")
+            os.remove(local_fname)
+        return self._add_test_callbacks(d, _test)
+
+    def test_invalid_credentials(self):
+        request = Request(url="ftp://127.0.0.1:%s/file.txt" % self.portNum,
+                meta={"ftp_user": self.username, "ftp_password": 'invalid'})
+        d = self.download_handler.download_request(request, None)
+
+        def _test(r):
+            self.assertEqual(r.type, ConnectionLost)
+        return self._add_test_callbacks(d, errback=_test)
